﻿using System.Collections.Generic;
using System.Linq;
using SmartStore.Core.Domain.Customers;
using SmartStore.Core.Domain.Security;
using SmartStore.Core.Security;

namespace SmartStore.WebApi.Security
{
    public static class WebApiPermissions
    {
        public const string Self = "webapi";
        public const string Read = "webapi.read";
        public const string Update = "webapi.update";
        public const string Create = "webapi.create";
        public const string Delete = "webapi.delete";
    }

    public class WebApiPermissionProvider : IPermissionProvider
    {
        public IEnumerable<PermissionRecord> GetPermissions()
        {
            var permissionSystemNames = PermissionHelper.GetPermissions(typeof(WebApiPermissions));
            var permissions = permissionSystemNames.Select(x => new PermissionRecord { SystemName = x });

            return permissions;
        }

        public IEnumerable<DefaultPermissionRecord> GetDefaultPermissions()
        {
            return new[]
            {
                new DefaultPermissionRecord
                {
                    CustomerRoleSystemName = SystemCustomerRoleNames.Administrators,
                    PermissionRecords = new[]
                    {
                        new PermissionRecord { SystemName = WebApiPermissions.Self }
                    }
                }
            };
        }
    }
}